package com.poc.web.controller;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.poc.web.entity.Employee;
import com.poc.web.model.Emp;
import com.poc.web.service.EmployeeService;

import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@RestController
@RequestMapping(path = "/employees")
@Validated
public class EmployeeController {
	@Autowired
	private EmployeeService employeeService;

	@GetMapping(path = "/")
	public ResponseEntity<Flux<Emp>> findAll() {
		return new ResponseEntity<>(employeeService.findAll(), HttpStatus.OK);
	}

	@PostMapping(produces = "application/json", consumes = "application/json", path = "/save")
	public ResponseEntity<Mono<Void>> saveEmployee(@Valid @RequestBody Emp emp) {
		return new ResponseEntity<>(employeeService.save(emp), HttpStatus.OK);
	}

	@GetMapping(produces = "application/json", consumes = "application/json", path = "/empId/{empId}")
	public ResponseEntity<Mono<Emp>> findByEmpId(@Valid @PathVariable Long empId) {
		return new ResponseEntity<>(employeeService.findByEmpId(empId), HttpStatus.OK);
	}

	@GetMapping(produces = "application/json", consumes = "application/json", path = "/id/{id}")
	public ResponseEntity<Mono<Emp>> findById(@Valid @PathVariable String id) {
		return new ResponseEntity<>(employeeService.findById(id), HttpStatus.OK);
	}

	@GetMapping(produces = "application/json", consumes = "application/json", path = "/sal/{sal}")
	public ResponseEntity<Flux<Emp>> findBySal(@Valid @PathVariable Double sal) {
		return new ResponseEntity<>(employeeService.findBySalary(sal), HttpStatus.OK);
	}

	@DeleteMapping(produces = "application/json", consumes = "application/json", path = "/delete/{empId}")
	public ResponseEntity<Mono<Long>> deleteByEmpId(@PathVariable Long empId) {
		return new ResponseEntity<>(employeeService.deleteByEmpId(empId), HttpStatus.OK);
	}

}
