package com.poc.web.service;

import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.poc.web.entity.Employee;
import com.poc.web.mapper.EmployeeMapper;
import com.poc.web.model.Emp;
import com.poc.web.repository.EmpMongoRepository;

import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class EmployeeServiceImpl implements EmployeeService {

	@Autowired
	private EmpMongoRepository empMongoRepository;
	@Autowired
	private EmployeeMapper employeeMapper;

	@Override
	public Mono<Emp> findByEmpId(Long empId) {
		return Mono.justOrEmpty((Emp) employeeMapper.mapEntityToPojo(empMongoRepository.findByEmpId(empId)));
	}

	@Override
	public Mono<Emp> findById(String id) {
		return Mono.just(
				(Emp) employeeMapper.mapEntityToPojo(empMongoRepository.findById(id).defaultIfEmpty(new Employee())));
	}

	@Override
	public Flux<Emp> findAll() {
		Flux<Employee> list=empMongoRepository.findAll();
		Flux<Emp> empList = list.map(e ->employeeMapper.mapEntityToPojo(e)).map(e -> (Emp) e);

		return empList;

	}

	@Override
	public Mono<Void> save(Emp emp) {
		Employee employee = (Employee) employeeMapper.mapPojoToEntity(emp);
		return empMongoRepository.save(employee).then();
	}

	@Override
	public Flux<Emp> findBySalary(Double sal) {
		Flux<Employee> stream = empMongoRepository.findBySalary(sal);
		Flux<Emp> empList = stream.map(e -> employeeMapper.mapEntityToPojo(e)).map(e -> (Emp) e);

		return empList;
	}

	@Override
	public Mono<Long> deleteByEmpId(Long empId) {
		return empMongoRepository.deleteByEmployeeId(empId);
	}

}
